//     Main routines.
//
//                             Fernando Anton - Mitra / Spanish Lords
//                                                  fernando@ergos.es
#include "slint.hpp"
#include "slints.hpp"
#include <i86.h>
#include <stdlib.h>
#include <mem.h>

// Installation state.
int               KeybInstalled=0;     // Keyboard installed ?
int               ExitKeybInstalled=0; // atexit keyboard routine installed ?
// Interrupts stuff.
unsigned short    OldInt9Selector=0;   // Old PM interrupt address.
unsigned long     OldPInt9Offset=0;
unsigned short    OldInt9Segment=0;    // Old RM interrupt address.
unsigned short    OldRInt9Offset=0;
// The following vars are needed for a bi-modal routine. The RM interrupt
// must have access to the data area, so it must a be a low memory address.
// By the way, we must reserve a DOS zone of memory, and our keyboard datas
// must be placed on that zone of memory.
int               DosMem9Allocated=0;  // Low memory reserved ?
unsigned short    DosMem9Seg=0;        // Low memory address.
unsigned short    DosMem9Sel=0;
// Pointer to low memory where keyboard data are placed.
// This is only an array of booleans. Each key has an index in this array,
// and you can know when it is being pressed looking its state in this
// array. ( See SLKEYS.HPP for keys index ).
// There are two special index in this array too :
//  - 253 : Keyboard commands counter. It is possible to use it as a
//          KeyPressed flag
//  - 254 : Last keyboard command. It is used inside the interrupt handler.
unsigned char far *Teclas=NULL;

// This function sets the bi-modal keyboard interrupt.
int  InstallKeyboard()
{
  if (!KeybInstalled)    // WARNING. We don't wanna install twice.
  {
  // Get actual interrupt address.
     GetPMInt(9,OldInt9Selector,OldPInt9Offset); // Get PM interrupt address.
     GetRMInt(9,OldInt9Segment ,OldRInt9Offset); // Get RM interrupt address.
  // Reserve 4Kb of DOS memory.
     if (!DosMem9Allocated) // Again, we don't wanna alloc twice.
        if (!GetDOSMem(4096/16,DosMem9Seg,DosMem9Sel)) return(0);
     DosMem9Allocated=1;
  // Create pointer to the recently reserved DOS memory.
     Teclas=(unsigned char far*)MK_FP(DosMem9Sel,0);
     _fmemset(Teclas,0,256); // Clean keyboard data zone.
  // Copy RM interrupt to DOS memory.
     _fmemcpy(MK_FP(DosMem9Sel,256),MK_FP(FP_SEG(SLRInt9),FP_OFF(SLRInt9)),4096-256);
  // Install own PM interrupt.
     if (!SetPMInt(9,FP_SEG(SLInt9),FP_OFF(SLInt9)))
     {
        if (FreeDOSMem(DosMem9Sel)) DosMem9Allocated=0;
        Teclas=NULL;
        return(0);
     }
  // Install own RM interrupt.
     if (!SetRMInt(9,DosMem9Seg,256))
     {
        SetPMInt(9,OldInt9Selector,OldPInt9Offset);
        if (FreeDOSMem(DosMem9Sel)) DosMem9Allocated=0;
        Teclas=NULL;
        return(0);
     }
     KeybInstalled=1;
  }
// Always call to UnInstallKeyboard when going out the program.
  if (!ExitKeybInstalled)
  {
     if (!atexit((void (*)())&UnInstallKeyboard)) ExitKeybInstalled=1;
  }
  return(1);
}

// Uninstall keyboard interrupt.
int  UnInstallKeyboard()
{
  if (KeybInstalled)  // Only if keyboard it's installed.
  {
  // Restore PM & RM interrupts.
     if (!SetPMInt(9,OldInt9Selector,OldPInt9Offset)) return(0);
     if (!SetRMInt(9,OldInt9Segment,OldRInt9Offset)) return(0);
  // Free DOS memory.
     if (!FreeDOSMem(DosMem9Sel)) return(0);
     DosMem9Allocated=0;
  // Non-installed at all.
     KeybInstalled=0;
     Teclas=NULL;
  }
  return(1);
}

